﻿using Dam.Core;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace Dam
{
    // A layer that updates its X position so it's always visible on the screen.
    public class BackgroundLayer
    {
        // Level of 0 = no parallax. 0.5 = half camera movement...
        public float ParallaxLevel { get; set; }
        public float MoveSpeedX { get; set; }
        public float realX;
        public int X => (int)realX;

        public float PreviousCameraPositionX { get; set; }

        public Texture2D Graphic { get; set; }
        
        // A wall is a rectangle, with additional shapes placed on front.
        // As the wall is built along, old shapes are turned into graphics-only (written to a sprite...)
        // And MinX is extended as the baseline for new objects.
        // Maybe draw a grey colour between MinX and MaxX to make it look a little nicer?

        public BackgroundLayer()
        {
            Reset();
        }

        public void Reset()
        {
            realX = -50;
        }

        public void Update(GameTime gameTime, float cameraOffset)
        {
            var cameraMovedBy = cameraOffset - PreviousCameraPositionX;

            realX += cameraMovedBy * ParallaxLevel;
            realX += MoveSpeedX;

            if (X > cameraOffset + DamGame.BASE_GAME_RES_WIDTH + DamGame.BASE_GAME_RES_WIDTH)
            {
                realX -= DamGame.BASE_GAME_RES_WIDTH;
            }
            if (X < cameraOffset + DamGame.BASE_GAME_RES_WIDTH)
            {
                realX += DamGame.BASE_GAME_RES_WIDTH;
            }

            PreviousCameraPositionX = cameraOffset;
        }
    }
}
