﻿using Dam.Controllers;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using System;

namespace Dam.Core
{
    public class DamGame : Game
    {
        GraphicsDeviceManager graphics;
        
        DamGameController gameController;
        DamSoundController soundController;
        DamGraphicsController graphicsController;
        DamInputController inputController;
        CameraController cameraController;

        public const int BASE_GAME_RES_WIDTH = 800;
        public const int BASE_GAME_RES_HEIGHT = 600;

        public DamGame()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
            IsFixedTimeStep = false;
            
            // Set the overall size of the game.
            graphics.PreferredBackBufferWidth = BASE_GAME_RES_WIDTH;
            graphics.PreferredBackBufferHeight = BASE_GAME_RES_HEIGHT;
            graphics.PreferredDepthStencilFormat = DepthFormat.Depth24Stencil8; // Enables stencil buffer...
            graphics.ApplyChanges();
        }
        
        protected override void Initialize()
        {
            Console.WriteLine("Loading Dam...");

            inputController = new DamInputController();
            soundController = new DamSoundController();
            cameraController = new CameraController();
            graphicsController = new DamGraphicsController(this, GraphicsDevice, cameraController);
            gameController = new DamGameController(this, graphicsController, inputController, cameraController);

#if EDITOR_MODE
            Console.WriteLine("Editor enabled.");
#endif

#if FAST_MODE
            Console.WriteLine("Fast mode enabled.");
#endif

            base.Initialize();

            gameController.SetState(DamGameController.MainMenu);
            Console.WriteLine("Dam loaded (main menu)");
        }
        
        protected override void LoadContent()
        {
            // Load graphics.
            graphicsController.LoadContent(Content);
            graphicsController.AfterLoad();

            // Load audio.
            soundController.LoadContent(Content);

            base.LoadContent();
        }

        protected override void Update(GameTime gameTime)
        {
            inputController.UpdateControlStates();
            cameraController.Update(gameTime);
            gameController.Update(gameTime);

            if(gameController.SwappingStates)
            {
                SuppressDraw();
            }

            base.Update(gameTime);
        }
        
        protected override void Draw(GameTime gameTime)
        {
            graphicsController.Draw();
            
            base.Draw(gameTime);
        }
    }
}
