﻿using Dam.Controllers;
using Dam.Core;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using System;

namespace Dam
{
    public class HeroGraphic
    {
        public Hero Hero { get; set; }
        public Texture2D Graphic { get; }

        public HeroGraphic(GraphicsDevice gd, Hero t)
        {
            Hero = t;
            Graphic = DamGraphicsController.Hero.GetRandomTexture();
        }
    }

    public class Hero
    {
        public const int MAX_PEBBLES = 10;
        public const int START_PEBBLES = 4;
        public const int MAX_HEALTH = 5;

        public const int HERO_WIDTH = 45;
        public const int HERO_HEIGHT = 75;

        public const float JUMP_MIN_FALL_PLAY_SOUND = 2f;

        public const int MOVE_THRUST = 4;
        public const int JUMP_THRUST = -32;

        public enum FaceDirection { Left, Right }

        public int X => (int)realX;
        public int Y => (int)realY;

        public FaceDirection Direction { get; set; }

        private float gravity = 1.5f;
        private float realY;
        private float velocityY;
        private float realX;
        public float velocityX;
        private float frictionX = 0.72f;
        
        public TimeSpan AnimationTimer { get; set; }
        public TimeSpan WalkingFrameTime { get; set; }
        public int CurrentFrame { get; set; }
        
        public bool IsGrounded { get; set; }

        public bool WaitingForLevelToStart { get; set; }
        public bool CanInteract => !WaitingForLevelToStart;

        private CameraController CameraController { get; }

        public Hero(CameraController cc, int x, int y)
        {
            CameraController = cc;

            Reset(x, y, -1, true);
        }

        public void Reset(int x, int y, int healthAtLevelStart, bool newGame)
        {
            realX = x;
            realY = y;
            velocityX = 0;
            velocityY = 0;

            CurrentFrame = 0;
            WaitingForLevelToStart = true;
        }

        public void MoveLeft()
        {
            if (CanInteract)
            {
                velocityX -= MOVE_THRUST;
                Direction = FaceDirection.Left;
            }
        }

        public void MoveRight()
        {
            if (CanInteract)
            {
                velocityX += MOVE_THRUST;
                Direction = FaceDirection.Right;
            }
        }
        
        public void TryJump()
        {
            if (CanInteract)
            {
                if (IsGrounded && velocityY >= 0)
                {
                    velocityY = JUMP_THRUST;

                    DamSoundController.Play(DamSoundController.RisingBoop);
                }
            }
        }
        
        public void Update(GameTime gt, DamGameController gc)
        {
            velocityX *= frictionX;
            velocityY += gravity;

            if (Math.Abs(velocityX) < 0.2f)
            {
                velocityX = 0;
            }

            var currentY = Y;
            var newY = (int)(realY + velocityY);

            IsGrounded = false;
            
            if (IsGrounded)
            {
                // Only apply velocity if it's going up.
                if (velocityY < 0)
                {
                    realY += velocityY;
                }
                else
                {
                    if (velocityY > JUMP_MIN_FALL_PLAY_SOUND)
                    {
                        DamSoundController.Play(DamSoundController.LowBoop);
                    }

                    velocityY = 0;
                }
            }
            else
            {
                realY += velocityY;
            }

            realX += velocityX;

            // left wall check
            if (realX < CameraController.CurrentPosition.X)
            {
                realX = CameraController.CurrentPosition.X;
                velocityX = 0;
            }
            // right wall check
            else if (realX + Hero.HERO_WIDTH > DamGame.BASE_GAME_RES_WIDTH + CameraController.CurrentPosition.X)
            {
                realX = DamGame.BASE_GAME_RES_WIDTH + CameraController.CurrentPosition.X - Hero.HERO_WIDTH;
                velocityX = 0;
            }
            // Dam check.
            else if (realX + HERO_WIDTH > Wall.WALL_INITIAL_X - Wall.WALL_INITIAL_WIDTH)
            {
                //realX = gc.currentWall.MinX - HERO_WIDTH;
                realX = Wall.WALL_INITIAL_X - Wall.WALL_INITIAL_WIDTH - HERO_WIDTH;
                velocityX = 0;
            }

            // Floor check
            if (realY >= DamGame.BASE_GAME_RES_HEIGHT - DamGameController.FLOOR_DISTANCE_FROM_BOTTOM_OF_SCREEN)
            {
                realY = DamGame.BASE_GAME_RES_HEIGHT - DamGameController.FLOOR_DISTANCE_FROM_BOTTOM_OF_SCREEN;
                IsGrounded = true;

                if (velocityY > JUMP_MIN_FALL_PLAY_SOUND)
                {
                    DamSoundController.Play(DamSoundController.LowBoop);
                }

                velocityY = 0;
            }
        }
    }
}
