﻿using Dam.Controllers;
using Dam.Core;
using Microsoft.Xna.Framework;
using Spl.Core;
using System;
using System.Collections.Generic;
using System.Linq;

namespace Dam
{
    public class Wall
    {
        // Leftmost wall boundary.
        public int MinX => Segments.Min(s => s.X);
        
        public const int WALL_INITIAL_X = -100; // How far is MaxX from the rightmost position on screen.
        public const int WALL_INITIAL_WIDTH = 90;
        public const int WALL_HEIGHT = 500; // Original: 500

        public const int WALL_BOTTOM_Y = DamGame.BASE_GAME_RES_HEIGHT - DamGameController.FLOOR_DISTANCE_FROM_BOTTOM_OF_SCREEN;
        public const int WALL_TOP_Y = WALL_BOTTOM_Y - Wall.WALL_HEIGHT;

        public const int NUM_SEGMENTS = 10;
        public const int SEGMENT_HEIGHT = WALL_HEIGHT / NUM_SEGMENTS;
        public WallSegment[] Segments = new WallSegment[NUM_SEGMENTS];

        public Crack[] Cracks { get; set; }

        // A wall is a rectangle, with additional shapes placed on front.
        // As the wall is built along, old shapes are turned into graphics-only (written to a sprite...)
        // And MinX is extended as the baseline for new objects.
        // Maybe draw a grey colour between MinX and MaxX to make it look a little nicer?

        public Wall()
        {
            Cracks = new Crack[NUM_SEGMENTS];
            for(int i = 0; i < NUM_SEGMENTS; i++)
            {
                Cracks[i] = new Crack { SegmentIdx = i, X = WALL_INITIAL_X - WALL_INITIAL_WIDTH };
            }

            Reset();
        }

        public void Reset()
        {
            foreach(var c in Cracks)
            {
                c.IsActive = false;
            }

            for(int i = 0; i < Segments.Length; i++)
            {
                Segments[i] = new WallSegment
                {
                    X = WALL_INITIAL_X - WALL_INITIAL_WIDTH
                };
            }
        }

        public void Update(GameTime gameTime)
        {
            foreach(var c in Cracks)
            {
                if(c.IsActive)
                {
                    c.Update(gameTime);
                }
            }
        }

        public void AddCrack()
        {
            List<int> validInts = Enumerable.Range(0, Wall.NUM_SEGMENTS).ToList();

            for(int i = NUM_SEGMENTS - 1; i >= 0; i--)
            {
                if (Cracks[i].IsActive)
                {
                    validInts.RemoveAt(i);
                }
            }

            if(validInts.Any())
            {
                var segmentIdx = validInts.RandomElement();
                var segment = Segments[segmentIdx];

                var newX = segment.X;
                var newY = WallSegment.GetSegmentY(segmentIdx);

                Cracks[segmentIdx].Reset(newX);

                DamSoundController.Play(DamSoundController.WallBreak);
            }
            else
            {
                Console.WriteLine("Tried to create crack, but all spots taken...");
            }
        }
    }
}
