﻿using Dam.Controllers;
using Dam.Core;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using System;

namespace Dam
{
    public class WaterGraphic
    {
        public Water Water { get; set; }
        public Texture2D Graphic { get; }
        public bool Visible { get; set; }

        public WaterGraphic(GraphicsDevice gd, Water w)
        {
            Water = w;
            Graphic = DamGraphicsController.Water.GetRandomTexture();
            Visible = false;
        }
    }

    public class Water
    {
        public int X => (int)realX;
        public int Y => (int)realY;

        public const int WIDTH = 35;
        public const int HEIGHT = 35;

        private float gravity = 1.5f;
        public float realX;
        public float realY;
        public float velocityX;
        public float velocityY;

        public float rotateSpeed;
        public float rotation;

        public bool isOnWall;

        public TimeSpan TimeAlive { get; set; }

        public bool markForDeletion;
        public bool markForNotBounce;

        public float VelocityMag
        {
            get
            {
                return (float)Math.Sqrt(Math.Pow(velocityX, 2) + Math.Pow(velocityY, 2));
            }
        }

        public float VelocityDir
        {
            get
            {
                return (float)Math.Atan2(velocityY, velocityX);
            }
        }

        public Vector2 Velocity
        {
            get
            {
                var angle = Math.Atan2(velocityY, velocityX);
                return new Vector2((float)Math.Cos(angle), (float)Math.Sin(angle));
            }
        }

        public Water(int x, int y, bool isLeft)
        {
            Reset(x, y, isLeft);
        }

        public void Reset(int x, int y, bool isLeft)
        {
            realX = x + DamGameController.random.Next(0, 100) / 25f;
            realY = y + DamGameController.random.Next(0, 100) / 25f;
            velocityX = DamGameController.random.Next(14, 18) / 1f * (isLeft ? -1 : 1);
            velocityY = DamGameController.random.Next(-20, -15);

            // rotate, random speed random direction.
            rotateSpeed = DamGameController.random.Next(4, 7) * (DamGameController.random.Next(0, 2) == 0 ? -1 : 1) / 50f;

            TimeAlive = TimeSpan.Zero;

            framesSinceLargeVelocity = 0;
            smallVelocity = 5f;

            markForDeletion = false;
            markForNotBounce = false;
        }

        int framesSinceLargeVelocity;
        float smallVelocity;

        public void Update(GameTime gt, DamGameController gc)
        {
            if (markForNotBounce)
            {
                rotateSpeed *= 0.85f;
            }

            rotation += rotateSpeed;

            if (Math.Abs(velocityY) > smallVelocity)
            {
                framesSinceLargeVelocity = 0;
            }
            framesSinceLargeVelocity++;
            if (framesSinceLargeVelocity > 10)
            {
                markForNotBounce = true;
            }
            if (framesSinceLargeVelocity > 60)
            {
                markForDeletion = true;
            }

            TimeAlive += gt.ElapsedGameTime;

            var currentY = Y;
            var newY = (int)(realY + velocityY);

            const float FLOOR_FRICTION = 0.7f;
            const float BOUNCE = 0;

            velocityY += gravity;
            
            // Floor check
            if (realY >= DamGame.BASE_GAME_RES_HEIGHT - DamGameController.FLOOR_DISTANCE_FROM_BOTTOM_OF_SCREEN)
            {
                realY = DamGame.BASE_GAME_RES_HEIGHT - DamGameController.FLOOR_DISTANCE_FROM_BOTTOM_OF_SCREEN;
                velocityX *= FLOOR_FRICTION;
                velocityY *= markForNotBounce ? 0 : -BOUNCE;
            }

            if (Math.Abs(velocityX) < 0.5f)
            {
                velocityX = 0;
            }

            realX += velocityX;
            realY += velocityY;
        }
    }
}
